﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.IntegerVectorEncoding;
using HeuristicLab.Optimization.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.BinPacking.Interfaces;

namespace HeuristicLab.Encodings.PackingEncoding.MultiComponentVector {
  [Item("Multi component tabu maker", "Declares a given single grouping move as tabu, by adding its attributes to the tabu list and also store the solution quality or the move quality (whichever is better).")]
  [StorableClass]
  public class MultiComponentVectorTabuMaker : TabuMaker, IMultiComponentVectorMoveOperator{
    public ILookupParameter<IPackingMove> PackingMoveParameter {
      get { return (ILookupParameter<IPackingMove>)Parameters["PackingMove"]; }
    }
    public ILookupParameter<MultiComponentVectorEncoding> MultiComponentVectorParameter {
      get { return (ILookupParameter<MultiComponentVectorEncoding>)Parameters["MultiComponentVector"]; }
    }

    [StorableConstructor]
    protected MultiComponentVectorTabuMaker(bool deserializing) : base(deserializing) { }
    protected MultiComponentVectorTabuMaker(MultiComponentVectorTabuMaker original, Cloner cloner) : base(original, cloner) { }
    public MultiComponentVectorTabuMaker()
      : base() {
        Parameters.Add(new LookupParameter<IPackingMove>("PackingMove", "The move to evaluate."));
      Parameters.Add(new LookupParameter<MultiComponentVectorEncoding>("MultiComponentVector", "The solution to evaluate."));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new MultiComponentVectorTabuMaker(this, cloner);
    }

    protected override IItem GetTabuAttribute(bool maximization, double quality, double moveQuality) {
      IPackingMove move = PackingMoveParameter.ActualValue;
      MultiComponentVectorEncoding solution = MultiComponentVectorParameter.ActualValue;
      double baseQuality = moveQuality;
      if (maximization && quality > moveQuality || !maximization && quality < moveQuality) 
        baseQuality = quality; // we make an uphill move, the lower bound is the solution quality
                              
      MultiComponentVectorMove mcvMove = move as MultiComponentVectorMove;
      if (mcvMove != null)
        return mcvMove.GetAttribute(baseQuality);
      else return solution;
    }
  }
}
